local addons_loaded = false;

local selectedIndex = nil;
local addons = {};

function CleanUIAddons_OnLoad(self)    
    -- addon config
	local addons = getglobal("CUIConfigAddons");
	addons.name = CONFIG_ADDONS_TITLE;
	addons.parent = "CleanUI";
    addons.addonname = "CleanUI";
    InterfaceOptions_AddCategory(addons);
end

function CleanUIAddons_HookEvents()
    hooksecurefunc(InterfaceOptionsFrame, "Show", CleanUIAddons_InterfaceOptionsFrame_Show);
    hooksecurefunc(InterfaceOptionsFrame, "Hide", CleanUIAddons_InterfaceOptionsFrame_Hide);
end

function CleanUIAddons_InterfaceOptionsFrame_Show()
    CleanUIAddons_Init();
end

function CleanUIAddons_InterfaceOptionsFrame_Hide()
end

function CleanUIAddons_OnClick(self, button, down)
    PlaySound("ACTIONBARBUTTONDOWN");
            
    selectedIndex = self.index;	
    
	local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(addons[selectedIndex]);
	local author = GetAddOnMetadata(name, "Author");
	local version = GetAddOnMetadata(name, "Version");
	
	local infoFrame = _G["CUIConfigAddons_Info"];
	infoFrame.title:SetText(title);
	infoFrame.notes:SetText(notes);
	infoFrame.author:SetText(author);
	infoFrame.version:SetText(version);
	
	if (not infoFrame:IsShown()) then
	    infoFrame:Show();
	end
end

function CleanUIAddons_OnEnter(self)
    --GameTooltip_SetDefaultAnchor(GameTooltip, self);
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    CleanUIAddons_CreateTooltip(GameTooltip, addons[self.index]);
end

function CleanUIAddons_CreateTooltip(tooltip, addonname)
	local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(addonname);
	local author = GetAddOnMetadata(name, "Author");
	local version = GetAddOnMetadata(name, "Version");
    tooltip:SetText("|cffffffff"..title.."|r");
    tooltip:AddDoubleLine("|c333399ff"..CONFIG_ADDONS_AUTHOR.."|r", author, nil, nil, nil, nil, nil, nil);
    tooltip:AddDoubleLine("|c333399ff"..CONFIG_ADDONS_VERSION.."|r", version, nil, nil, nil, nil, nil, nil);
    
    if (notes and strlen(notes) > 0) then
        tooltip:AddLine(" ", nil, nil, nil, 1);
        tooltip:AddLine("|cffffffff"..notes.."|r", nil, nil, nil, 1);
    end
    
    tooltip:AddLine(" ", nil, nil, nil, 1);
    
    -- Abhngigkeiten		    
    local allDependenciesAvailable = true;
    local hasDependencies = false;
    
    tooltip:AddLine("|c333399ff"..CONFIG_ADDONS_DEPENDENCIES.."|r", nil, nil, nil, 1);
    
    local dependencies = GetAddOnDependencies(name);
    if (dependencies) then
        hasDependencies = true;
        for i=1, select("#", dependencies) do
            local dep_name = select(i,dependencies);
	        local _, dep_title = GetAddOnInfo(dep_name);
            
            tooltip:AddLine("|cffffffff  - "..dep_title.."|r", nil, nil, nil, 1);
            
            local depIsLoaded = IsAddOnLoaded(dep_name);
            if (not depIsLoaded) then
                allDependenciesAvailable = false;
                break;
            end
        end
    end
    
    if (not hasDependencies) then
        tooltip:AddLine("|cff00aa00"..CONFIG_ADDONS_DEPENDENCIES_NONE.."|r", nil, nil, nil, 1);
    end
    
    if (not allDependenciesAvailable) then
        tooltip:AddLine(" ", nil, nil, nil, 1);
        tooltip:AddLine("|cffaa0000"..CONFIG_ADDONS_MISSING_DEPENDENCY.."|r", nil, nil, nil, 1);
    end
    
    UpdateAddOnMemoryUsage();
    local memusage = GetAddOnMemoryUsage(addonname);
    tooltip:AddLine("|c333399ff"..CONFIG_ADDONS_MEMORY_USAGE.."|r", nil, nil, nil, 1);  
    tooltip:AddLine("|cffffffff  "..string.format("%.2f kB", memusage).."|r", nil, nil, nil, 1); 
    
    tooltip:Show();
end

function CleanUIAddons_OnLeave(self)
	GameTooltip:Hide();
end

function CleanUIAddons_Init()   
    if (addons_loaded) then
        return;
    end
	
	-- load addons
    local count = GetNumAddOns();
    
    local name, dependencies;    
    for i = 1, count do
        name = GetAddOnInfo(i);
        if (name) then
            tinsert(addons, name);
        end
    end
    
	table.sort(addons, function(a,b) return a<b end);

    -- create gui
	HybridScrollFrame_OnLoad(CUIConfigAddons.scrollFrame);
	CUIConfigAddons.scrollFrame.update = CleanUIAddons_OnUpdate;
	HybridScrollFrame_CreateButtons(CUIConfigAddons.scrollFrame, "CUIAddonButtonTemplate");
	
    CUIConfigAddons.scrollFrame:SetFrameStrata("HIGH");
	
	local borderframe = _G["CUIAddonsBorderFrame"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUIAddonsBorderFrame", CUIConfigAddons.scrollFrame);
	end
	borderframe:SetPoint("TOPLEFT", -10, 5);
	borderframe:SetPoint("BOTTOMRIGHT", 25, -5);
    borderframe:SetFrameStrata("BACKGROUND");

	borderframe:SetBackdrop( { 
        bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
        edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    borderframe:SetBackdropColor(0.0, 0.0, 0.0, 1.0);	

	CUIConfigAddons_Info:SetBackdrop( { 
        bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
        edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    CUIConfigAddons_Info:SetBackdropColor(0.0, 0.0, 0.0, 1.0);	

    addons_loaded = true;
end

function CleanUIAddons_OnUpdate()	
    if (not addons_loaded) then
        return;
    end
	
	CUIConfigAddonsHighlightFrame:Hide();
	
	local buttons = CUIConfigAddons.scrollFrame.buttons;
	
	local numEntries = #addons;
	local numButtons = #buttons;
		    
	local scrollOffset = HybridScrollFrame_GetOffset(CUIConfigAddons.scrollFrame);
	local buttonHeight = buttons[1]:GetHeight();
	local displayedHeight = 0;
	
	local addon;
	local name, title, notes, enabled, loadable, reason, security;
	
	for i=1, numButtons do
	
		local addonIndex = i + scrollOffset;
		addon = buttons[i];
		
		if ( addonIndex <= numEntries ) then
		    name = addons[addonIndex];
		    name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(name);
		    
			addon:Show();
		    addon:SetText(title);
		    
		    addon.index = addonIndex;
		    addon.checkButtonActive:SetChecked(enabled);
		    if (loadable and not IsAddOnLoaded(name)) then
		        addon.loadButton:Show();
		    else
		        addon.loadButton:Hide();
		    end
		    
		    -- Abhngigkeiten		    
		    local allDependenciesAvailable = true;
		    
		    local dependencies = GetAddOnDependencies(name);
            if (dependencies) then
                for i=1, select("#", dependencies) do
                    local dep_name = select(i,dependencies);
                    
                    local depIsLoaded = IsAddOnLoaded(dep_name);
                    if (not depIsLoaded) then
                        allDependenciesAvailable = false;
                        break;
                    end
                    
                    local _, _, _, depEnabled = GetAddOnInfo(dep_name);
                    if (not depEnabled) then
                        allDependenciesAvailable = false;
                        break;
                    end
                end
            end
            
            if (allDependenciesAvailable) then
		        addon.missingDependency:Hide();
		        addon.checkButtonActive:Enable();
            else
		        addon.missingDependency:Show();
		        addon.checkButtonActive:Disable();
            end

            -- this isn't a header, hide the header textures
            addon:SetNormalTexture("");
            addon:SetHighlightTexture("");
            
            if (addonIndex == selectedIndex) then
                -- reposition highlight frames
                CUIConfigAddonsHighlightFrame:SetParent(addon);
                CUIConfigAddonsHighlightFrame:SetPoint("TOPLEFT", addon, "TOPLEFT", 0, 0);
                CUIConfigAddonsHighlightFrame:SetPoint("BOTTOMRIGHT", addon, "BOTTOMRIGHT", 0, 0);
		        CUIConfigAddonsHighlightFrame:Show();
            end
		else
			addon:Hide();
		end
		
		displayedHeight = displayedHeight + buttonHeight;
	end
	
	HybridScrollFrame_Update(CUIConfigAddons.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUIAddons_CheckButtonActiveClicked(self)
    local index = self:GetParent().index;
    local name = addons[index];
    
    if (self:GetChecked()) then
        EnableAddOn(name);
    else
        CleanUIAddons_DisableAddOn(name);
    end
end

function CleanUIAddons_DisableAddOn(name)
	for i=1, #addons do
	    local actName = addons[i];
	    
	    if (actName ~= name) then 
		    local dependencies = GetAddOnDependencies(actName);   
            if (dependencies) then
                for i=1, select("#", dependencies) do
                    local dep_name = select(i,dependencies);
                    if (dep_name == name) then
                        local _, _, _, depEnabled = GetAddOnInfo(actName);
                        if (depEnabled) then
                            CleanUIAddons_DisableAddOn(actName);
                        end
                    end
                end
            end
        end
    end

    DisableAddOn(name);
end

function CleanUIAddons_ButtonLoadClicked(self)
    local index = self:GetParent().index;
    local name = addons[index];
    
    LoadAddOn(name);
end