
-- temporary data
local item_profit = 0;

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;
    
-- big bag
local bigBagLabels = {}; 
local lastBigBagScrollPosition = nil;
local bigBagInitialized = false;   
local emptyDefaultItem = nil;

local SORT_ITEMS = {
    CONFIG_BIGBAG_SORTBY_NONE,
    CONFIG_BIGBAG_SORTBY_TYPE,
}

function CleanUI_BagsOnEvent(self, event, ...)

    if (event == "MERCHANT_SHOW") then
 		CleanUI_MerchantShow();
 	end
    
    if (event == "BANKFRAME_OPENED") then
        CleanUI_OpenAllBags();
    end
    
    if (event == "BAG_UPDATE") then
        CleanUI_BagUpdate();
    end
end

function CleanUI_BagsHookEvents()	
	if (CleanUIData.CHANGE_BAGS ~= 1) then
	    return;
	end
	
	hooksecurefunc("ContainerFrame_Update", function(frame) CleanUI_ContainerFrame_Update(frame) end);
	hooksecurefunc("BankFrameItemButton_Update", function(button) CleanUI_BankFrameItemButton_Update(button) end);
end

function CleanUI_InitBags()     
	if (CleanUIData.CHANGE_BAGS ~= 1) then
	    return;
	end
	
	-- Display the amount of copper that is added to the tooltip
	GameTooltip:HookScript("OnTooltipAddMoney",
		function(self, amount)
    		item_profit = amount;
		end);
    
    CUI_Bagpack_MinimapButton_Reposition();
		
    CleanUI_ReorganizeButtons();
    CleanUI_InitBigBag();
end

function CleanUI_ReorganizeButtons()
		
	if (MASQUE) then
		buttongroup = MASQUE:Group("CleanUI");
	end
	
	MainMenuBarBackpackButton:SetWidth(36);
	MainMenuBarBackpackButton:SetHeight(36);
    
	for i=0, 3, 1 do
		local button = getglobal("CharacterBag"..i.."Slot");
        button:SetWidth(24);
        button:SetHeight(24);
	end
	
    MainMenuBarBackpackButton:ClearAllPoints();
    MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", CleanUIActionButtonsArtFrame, "BOTTOMRIGHT", -7, 7);
    
    CharacterBag0Slot:ClearAllPoints();
    CharacterBag0Slot:SetPoint("RIGHT", MainMenuBarBackpackButton, "LEFT", -4, 0);
    CharacterBag1Slot:ClearAllPoints();
    CharacterBag1Slot:SetPoint("RIGHT", CharacterBag0Slot, "LEFT", -2, 0);
    CharacterBag2Slot:ClearAllPoints();
    CharacterBag2Slot:SetPoint("RIGHT", CharacterBag1Slot, "LEFT", -2, 0);
    CharacterBag3Slot:ClearAllPoints();
    CharacterBag3Slot:SetPoint("RIGHT", CharacterBag2Slot, "LEFT", -2, 0);
	
	if (MASQUE and buttongroup) then
		for i=0, 3, 1 do
			local button = getglobal("CharacterBag"..i.."Slot");
			local iconTexture = getglobal("CharacterBag"..i.."SlotIconTexture");
			iconTexture:SetDrawLayer("BACKGROUND");
			buttongroup:AddButton(button);
		end
		
		buttongroup:AddButton(MainMenuBarBackpackButton);
		local iconTexture = getglobal("MainMenuBarBackpackButtonIconTexture");
		iconTexture:SetDrawLayer("BACKGROUND");
	end
		
	CleanUI_MoveBagButtons();
end

function CleanUI_MoveBagButtons()	
    if (CleanUIData.BAGS_SHOWBACKBACKBUTTON == 1) then
        MainMenuBarBackpackButton:Show();
    else
        MainMenuBarBackpackButton:Hide();
    end

    if (CleanUIData.BAGS_MOVEBAGBUTTONS == 1) then
    
        local parent = ContainerFrame1;
    
        if (CleanUIData.USE_BIGBAG == 1) then
            parent = CleanUIBigBag;
        end
    
        CharacterBag0Slot:ClearAllPoints();
        CharacterBag0Slot:SetPoint("TOPRIGHT", parent, "BOTTOMRIGHT", -2, 0);
        
		for i=0, 3, 1 do
			local button = getglobal("CharacterBag"..i.."Slot");
			button:SetParent(parent);
		end
    else
        CharacterBag0Slot:ClearAllPoints();
        CharacterBag0Slot:SetPoint("RIGHT", MainMenuBarBackpackButton, "LEFT", -5, 0);
        
		for i=0, 3, 1 do
			local button = getglobal("CharacterBag"..i.."Slot");
			button:SetParent(MainMenuBarBackpackButton);
		end
    end
end

function CleanUI_ContainerFrame_Update(frame)
	local id = frame:GetID();
	local name = frame:GetName();
	
	local itemButton;
	local texture, itemCount, locked, quality, readable;
	local tooltipOwner = GameTooltip:GetOwner();
	
	for i=1, frame.size, 1 do
		local itemButton = _G[name.."Item"..i];
		local itemLink = GetContainerItemLink(id, itemButton:GetID());
		
		CleanUI_SetItemButtonBorderColor(itemLink, itemButton);
	end
end

function CleanUI_BankFrameItemButton_Update(itemButton)
	local itemLink = GetContainerItemLink(BANK_CONTAINER, itemButton:GetID());		
	CleanUI_SetItemButtonBorderColor(itemLink, itemButton);
end

function CleanUI_SetItemButtonBorderColor(itemLink, itemButton)

	local itemButtonBorder = itemButton:GetNormalTexture();
	
	if (not itemButtonBorder) then
	    return;
	end
	
	itemButtonBorder:SetDrawLayer("ARTWORK");
	
	itemButtonBorder:SetTexture("interface\\AddOns\\CleanUI\\skins\\containerbuttonborder");
	
	if (CleanUIData.BAGS_COLORIZE_ITEMS ~= 1) then
		itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
		return;
	end
	
    itemButtonBorder:Show();
    itemButtonBorder:SetAlpha(1);
    
	if (itemLink) then	
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
	
		if (itemType ~= "Quest" and itemRarity) then
			local r, g, b, hex = GetItemQualityColor(itemRarity);
			
			if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY == 1) then
			    r, g, b = 0.5, 0.5, 0.5;
			end
			
			itemButtonBorder:SetVertexColor(r, g, b);
		else
			itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
		end
	else
        itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
        itemButtonBorder:SetAlpha(0.3);
	end
end

function CleanUI_MerchantShow()

	-- auto repair
	if (not IsControlKeyDown() and CanMerchantRepair() == 1 and CleanUIData.BAGS_AUTOREPAIR == 1) then
		repairAllCost, canRepair = GetRepairAllCost();
		
		if (IsAltKeyDown()) then
			RepairAllItems(1);      -- guild
		else
			RepairAllItems();
		end
		
		if (repairAllCost > 0) then
 			DEFAULT_CHAT_FRAME:AddMessage(CUI_BAGS_ALL_ITEMS_REPAIRED..". ("..GetCoinTextureString(repairAllCost)..")", 1.0, 1.0, 0, 1, 10);
		end
	end
	
	local sum_item_profit = 0;

	-- auto sell grey items
	if (CleanUIData.BAGS_AUTOSELLGRAY == 1) then
		for id = 0, 4, 1 do
		 	local containerid = "ContainerFrame"..(id + 1);
 			local container = getglobal(containerid);
 	
 			if (container) then
 				local numberOfSlots = GetContainerNumSlots(id);
 			
 				for slot=1, numberOfSlots, 1 do
 					local itemLink = GetContainerItemLink(id, numberOfSlots - slot + 1);
 				
 					if (itemLink) then
 						local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
 		
 						if (itemRarity == 0) then
 							GameTooltip:SetBagItem(id, numberOfSlots - slot + 1);
 					
 							if (item_profit and item_profit > 0) then
 								sum_item_profit = sum_item_profit + item_profit;
 								DEFAULT_CHAT_FRAME:AddMessage(itemLink.. " "..CUI_BAGS_ALL_ITEMS_SOLD..". ("..GetCoinTextureString(item_profit)..")", 1.0, 1.0, 0, 1, 10);
 								UseContainerItem(id, numberOfSlots - slot + 1);
 							end
 						end
 					end
 				end
			end
		end
 	
 		if (sum_item_profit > 0) then
 			DEFAULT_CHAT_FRAME:AddMessage(CUI_BAGS_TOTAL_AMOUNT..": "..GetCoinTextureString(sum_item_profit), 1.0, 1.0, 0, 1, 10);
 		end
 	end
end

function CleanUI_OpenAllBags()
	if ( BankFrame:IsShown() ) then
		ToggleBag(5);
		ToggleBag(6);
		ToggleBag(7);
		ToggleBag(8);
		ToggleBag(9);
		ToggleBag(10);
		ToggleBag(11);
	end
end

function CUI_Bagpack_MinimapButton_Reposition()
	CUI_Bagpack_MinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT",
	                              52-(80*cos(CleanUIData.BAGPACK_MINIMAP_POS)), (80*sin(CleanUIData.BAGPACK_MINIMAP_POS))-52);
end

function CUI_Bagpack_MinimapButton_DraggingFrame_OnUpdate()
	local xpos,ypos = GetCursorPosition();
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

	xpos = xmin-xpos/UIParent:GetScale()+70;
	ypos = ypos/UIParent:GetScale()-ymin-70;

	CleanUIData.BAGPACK_MINIMAP_POS = math.deg(math.atan2(ypos,xpos));
	CUI_Bagpack_MinimapButton_Reposition();
end

function CleanUI_BagUpdate() 
    if (CleanUIData.USE_BIGBAG ~= 1 or not bigBagInitialized) then 
        return;
    end
    
    CleanUI_MoveBagItems(CleanUIBigBag);
end

function CleanUI_InitBigBag()    
    
    if (CleanUIData.USE_BIGBAG == 1) then
        CleanUIBigBag:Show();
    else
        CleanUIBigBag:Hide();
        return;
    end
    
    CleanUIData.BIGBAG_SHOW_EMPTY = 0;
    
    UIDropDownMenu_SetSelectedValue(CleanUIBigBagSortDropdown, 1);
    UIDropDownMenu_SetText(CleanUIBigBagSortDropdown, SORT_ITEMS[1]);
    
    CleanUIBigBagSortDropdownLeft:SetTexture();
    CleanUIBigBagSortDropdownMiddle:SetTexture();
    CleanUIBigBagSortDropdownRight:SetTexture();
    
    -- An-/Ausschalten deaktivieren
    for i=0, 3, 1 do
        local button = getglobal("CharacterBag"..i.."Slot");
        button:RegisterForClicks();
    end  
          
    hooksecurefunc("ContainerFrame_Update", function(frame) CleanUI_ContainerFrame_Update(frame) end);
    
    hooksecurefunc("OpenAllBags", CleanUI_ReinitBigBag);
    hooksecurefunc("OpenBackpack", CleanUI_OpenBackpack);
    hooksecurefunc("ToggleBackpack", CleanUI_ToggleBackpack);

    local bigbag = _G["CleanUIBigBag"];
    
    bigbag:SetBackdrop( { 
            bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
            edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
            insets = { left = 4, right = 4, top = 4, bottom = 4 }
        });
    bigbag:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
    
    local searchBoxBackground = _G["CleanUISearchBoxBackground"];
    if (not searchBoxBackground) then
        searchBoxBackground = CreateFrame("Frame", "CleanUISearchBoxBackground", BagItemSearchBox);
    
        searchBoxBackground:SetBackdrop( { 
                edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
                insets = { left = 4, right = 4, top = 4, bottom = 4 }
            });
        searchBoxBackground:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
    
        searchBoxBackground:ClearAllPoints(); 
        searchBoxBackground:SetPoint("TOPLEFT", -5, 1);
        searchBoxBackground:SetPoint("BOTTOMRIGHT", 0, -2);
    end
    
    bigBagInitialized = true;
end

function CleanUI_CloseAllBags()  
    for i=0, 4 do
        CloseBag(i);
    end
end

function CleanUI_ReinitBigBag()  
    CleanUI_MoveBagFeatureFrames(CleanUIBigBag);
    CleanUI_MoveBagItems(CleanUIBigBag);
    CleanUI_MoveBagButtons();
end

function CleanUI_OpenBackpack()  
    OpenBag(1);
    OpenBag(2);
    OpenBag(3);
    OpenBag(4);
    CleanUI_ReinitBigBag();
end

function CleanUI_ToggleBackpack() 
    if (IsBagOpen(0)) then
        CleanUI_OpenBackpack();
    end
end

function CleanUI_MoveBagFeatureFrames(frame)
    BagItemSearchBox:SetParent(frame);
    
    BagItemSearchBox:ClearAllPoints(); 
    BagItemSearchBox:SetPoint("TOPRIGHT", -4, -25);
    
    BagItemSearchBoxLeft:SetTexture(nil);
    BagItemSearchBoxRight:SetTexture(nil);
    BagItemSearchBoxMiddle:SetTexture(nil);
    
    BackpackTokenFrame:SetParent(frame);    
    BackpackTokenFrame:ClearAllPoints(); 
    BackpackTokenFrame:SetPoint("BOTTOMRIGHT", 5, -3);
    CleanUI_RemoveTextures(BackpackTokenFrame);
    
    ContainerFrame1MoneyFrame:SetParent(frame);    
    ContainerFrame1MoneyFrame:ClearAllPoints(); 
    ContainerFrame1MoneyFrame:SetPoint("BOTTOMLEFT", 10, 7);
end

function CleanUI_MoveBagItems(frame)
    
    -- hide blizzard container frames
    for i=1, 5 do
        name = "ContainerFrame"..i;
        container = _G[name];
        
        container:ClearAllPoints(); 
        container:SetPoint("LEFT", UIParent, "RIGHT", 10000, 0);
    end
    
    -- move items
    local MAX_FRAMEHEIGHT = 500;

    local ITEMSPERROW = 10;
    local ITEMSIZE = 40;

    local countrows = 0;
    
    for index, label in pairs(bigBagLabels) do
        label:Hide();
    end
    
    local sort = UIDropDownMenu_GetSelectedValue(CleanUIBigBagSortDropdown);
    
    emptyDefaultItem = nil;
    
    if (sort == 1) then
        countrows = CleanUIBigBag_AddItems(MAX_FRAMEHEIGHT, ITEMSPERROW, ITEMSIZE);
    elseif (sort == 2) then
        countrows = CleanUIBigBag_AddItemsByType(MAX_FRAMEHEIGHT, ITEMSPERROW, ITEMSIZE);
    end

    local frameheight = countrows * ITEMSIZE + 135;
    
    if (frameheight > MAX_FRAMEHEIGHT) then
        frameheight = MAX_FRAMEHEIGHT;
    end
    
    frame:SetHeight(frameheight);
    
    local innerheight = countrows * ITEMSIZE + 5;
    CleanUIBigBagInnerFrame:SetHeight(innerheight);
    
    -- check hidden
    lastBigBagScrollPosition = nil;
    CleanUIBigBag_OnUpdate(CleanUIBigBagScrollFrame:GetVerticalScroll());
end

function CleanUIBigBag_AddItems(MAX_FRAMEHEIGHT, ITEMSPERROW, ITEMSIZE)

    local row = 0;
    local column = 0;

    local name, container, bagType;

    for i=1, 5 do
        name = "ContainerFrame"..i;
        container = _G[name];
        
        if (container.size) then
            for i=1, container.size, 1 do        
                local itemButton = _G[name.."Item"..i];
                local itemLink = GetContainerItemLink(container:GetID(), itemButton:GetID());
                
                if (not itemLink) then
                    CleanUIBigBag_CollectEmptyItems(container:GetID(), itemButton);
                end
                
                if (itemLink or CleanUIData.BIGBAG_SHOW_EMPTY == 1) then
                    if (mod(column, ITEMSPERROW) == 0) then
                        row = row + 1;
                        column = 0;
                    end
                    
                    column = column + 1;
                    
                    itemButton:SetFrameStrata("HIGH");
                    
                    itemButton:ClearAllPoints(); 
                    itemButton:SetPoint("TOPLEFT", CleanUIBigBagInnerFrame, "TOPLEFT", (column-1) * ITEMSIZE + 5, (row-1) * ITEMSIZE * -1 - 3);
                else
                    itemButton:ClearAllPoints(); 
                    itemButton:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 10000, 0);
                end
            end
        end
    end

    return row;
end

function CleanUIBigBag_AddItemsByType(MAX_FRAMEHEIGHT, ITEMSPERROW, ITEMSIZE)

    local sortedItems = {};
    sortedItems["NONE"] = {};
    
    local name, container, type, subtable, label;

    for i=1, 5 do
        name = "ContainerFrame"..i;
        container = _G[name];
        if (container.size) then
            for i=1, container.size, 1 do     
                local itemButton = _G[name.."Item"..i];   
                local itemLink = GetContainerItemLink(container:GetID(), itemButton:GetID());
                
                type = "NONE"; 

                if (itemLink) then
                    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
                    type = itemType; 
                    
                    itemButton.cui_subtype = itemSubType..itemName;                
                end
                
                if (not itemLink) then
                    CleanUIBigBag_CollectEmptyItems(container:GetID(), itemButton);
                end
                
                subtable = sortedItems[type];
                
                if (not subtable) then
                    subtable = {};
                    sortedItems[type] = subtable;
                end
                
                table.insert(subtable, itemButton);
            end
        end
    end

    local row = 0;
    local column = 0;
    
    for type, subtable in pairs(sortedItems) do
        
        if (type ~= "NONE") then
    
            table.sort(subtable, function(a,b) return CleanUIBigBag_SortBySubtype(a, b) end);
        
            label = bigBagLabels[type];
            if (not label) then
                label = CleanUIBigBagInnerFrame:CreateFontString();            
                label:SetFontObject("GameFontNormal");
                label:SetJustifyH("LEFT");
                label:SetWidth(250);
                label:SetHeight(10);
                label:SetText(type..":");
                
                bigBagLabels[type] = label;
            end   
             
            label:Show();
            label:ClearAllPoints(); 
            label:SetPoint("TOPLEFT", CleanUIBigBagInnerFrame, "TOPLEFT", 10, (row-1) * ITEMSIZE * -1 - 21);
        end   
        
        for index, itemButton in pairs(subtable) do
            if (type ~= "NONE" or CleanUIData.BIGBAG_SHOW_EMPTY == 1) then
                if (mod(column, ITEMSPERROW) == 0) then
                    row = row + 1;
                    column = 0;
                end
                
                column = column + 1;
                
                itemButton:SetFrameStrata("HIGH");
                
                itemButton:ClearAllPoints(); 
                itemButton:SetPoint("TOPLEFT", CleanUIBigBagInnerFrame, "TOPLEFT", (column-1) * ITEMSIZE + 5, (row-1) * ITEMSIZE * -1 - 3);
            else
                itemButton:ClearAllPoints(); 
                itemButton:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 10000, 0);
            end
        end
        
        row = row + 1;
        column = 0;
    end
    
    return row - 1;
end

function CleanUIBigBag_CollectEmptyItems(containerid, itemButton)

--[[
BagType:
    0 = Unspecified (for bags it means any item, for items it means no special bag type)
    1 = Quiver
    2 = Ammo Pouch
    4 = Soul Bag 
    8 = Leatherworking Bag
    16 = Inscription Bag
    32 = Herb Bag
    64 = Enchanting Bag
    128 = Engineering Bag
    256 = Keyring
    512 = Gem Bag
    1024 = Mining Bag
    2048 = Unknown
    4096 = Vanity Pets
]]

    if (CleanUIData.BIGBAG_SHOW_EMPTY ~= 1) then        
        _, bagType = GetContainerNumFreeSlots(containerid);
        
        if (not emptyDefaultItem) then
            emptyDefaultItem = itemButton;
        end
    end
end

function CleanUIBigBag_SortBySubtype(a, b)
    if (a.cui_subtype < b.cui_subtype) then
        return true;
    end
    
    return false;
end

function CleanUIBigBag_OnMouseDown(frame, button)
    frame:StartMoving();
end

function CleanUIBigBag_OnMouseUp(frame, button)
    frame:StopMovingOrSizing();
end

function CleanUIBigBagSortDropdown_Initialize()
    for i = 1, #SORT_ITEMS do
        local info = UIDropDownMenu_CreateInfo();
        info.text = SORT_ITEMS[i];
        info.value = i;
        info.func = function()
                CleanUIBigBagSortDropdown_Select(i);
            end
        UIDropDownMenu_AddButton(info);
    end
end

function CleanUIBigBagSortDropdown_Select(i)
    UIDropDownMenu_SetText(CleanUIBigBagSortDropdown, SORT_ITEMS[i]);
    UIDropDownMenu_SetSelectedValue(CleanUIBigBagSortDropdown, i);
    CleanUI_MoveBagItems(CleanUIBigBag);
end

function CleanUIBigBag_OnUpdate(i)
    if (lastBigBagScrollPosition ~= nil and i == lastBigBagScrollPosition) then
        return;
    end
    
    local itemButton, point, relativeTo, relativePoint, xOfs, yOfs, y, diff, alpha;
    
    lastBigBagScrollPosition = i;

    local minPosition = i;
    local maxPosition = i + CleanUIBigBagScrollFrame:GetHeight();
    
    for i=1, 5 do
        name = "ContainerFrame"..i;
        container = _G[name];
        if (container.size) then
            for i=1, container.size, 1 do     
                itemButton = _G[name.."Item"..i];   
                
                itemButton:SetAlpha(1);
                itemButton:Show();
                
                point, relativeTo, relativePoint, xOfs, yOfs = itemButton:GetPoint(1);   
                      
                y = -yOfs;
                
                if (y + 20 < minPosition) then
                    itemButton:Hide();
                elseif (y < minPosition) then
                    diff = minPosition-y;
                    alpha = 1 - diff/20;
                    itemButton:SetAlpha(alpha);
                end
                
                if (y + 20 > maxPosition) then
                    itemButton:Hide();
                elseif (y + itemButton:GetHeight() > maxPosition) then
                    diff = y + itemButton:GetHeight() - maxPosition;
                    alpha = 1 - diff/20;
                    itemButton:SetAlpha(alpha);
                end  
            end 
        end 
    end 
    
    if (emptyDefaultItem) then
        emptyDefaultItem:Show();
        emptyDefaultItem:SetAlpha(1);
                    
        emptyDefaultItem:SetFrameStrata("HIGH");
        
        emptyDefaultItem:ClearAllPoints(); 
        emptyDefaultItem:SetPoint("TOPLEFT", CleanUIBigBagScrollFrame, "BOTTOMLEFT", 4, -12);
    end
end








