local inv_slots = {
	(GetInventorySlotInfo("HeadSlot")),
	(GetInventorySlotInfo("NeckSlot")),
	(GetInventorySlotInfo("ShoulderSlot")),
	(GetInventorySlotInfo("BackSlot")),
	(GetInventorySlotInfo("ChestSlot")),
	(GetInventorySlotInfo("WristSlot")),
	(GetInventorySlotInfo("HandsSlot")),
	(GetInventorySlotInfo("WaistSlot")),
	(GetInventorySlotInfo("LegsSlot")),
	(GetInventorySlotInfo("FeetSlot")),
	(GetInventorySlotInfo("Finger0Slot")),
	(GetInventorySlotInfo("Finger1Slot")),
	(GetInventorySlotInfo("Trinket0Slot")),
	(GetInventorySlotInfo("Trinket1Slot")),
	(GetInventorySlotInfo("MainHandSlot")),
	(GetInventorySlotInfo("SecondaryHandSlot")),
	(GetInventorySlotInfo("RangedSlot")),
}

local inv_slot_buttons = {
	"CharacterHeadSlot",
	"CharacterNeckSlot",
	"CharacterShoulderSlot",
	"CharacterBackSlot",
	"CharacterChestSlot",
	"CharacterWristSlot",
	"CharacterHandsSlot",
	"CharacterWaistSlot",
	"CharacterLegsSlot",
	"CharacterFeetSlot",
	"CharacterFinger0Slot",
	"CharacterFinger1Slot",
	"CharacterTrinket0Slot",
	"CharacterTrinket1Slot",
	"CharacterMainHandSlot",
	"CharacterSecondaryHandSlot",
	"CharacterRangedSlot",
}

function CleanUI_CharacterFrameOnEvent(self, event, ...)
    if (CleanUIData.CHANGE_CHARACTERFRAME ~= 1) then 
        return;
    end
    
	if (event == "UNIT_INVENTORY_CHANGED") then
		local unit = ...;
		if (unit == "player") then
		    CleanUI_UpdateColoredBorders();
        end
	end
end

function CleanUI_InitCharacterFrame()
    for i = 1, #inv_slots do	
        local buttonname = inv_slot_buttons[i];
        local button = _G[buttonname];
        
        local itemButtonBorder = _G[buttonname.."NormalTexture"];        
        itemButtonBorder:SetTexture();
        
        local texture = button:CreateTexture(buttonname.."CUI_Border", OVERLAY);
        texture:SetAllPoints(itemButtonBorder);        
        texture:SetTexture("interface\\AddOns\\CleanUI\\skins\\containerbuttonborder");
    end
    
    CleanUI_UpdateColoredBorders();
end

function CleanUI_CharacterFrameHookEvents()
    if (CleanUIData.CHANGE_CHARACTERFRAME ~= 1) then 
        return;
    end 
end

function CleanUI_UpdateColoredBorders()

    for i = 1, #inv_slots do	
        local buttonname = inv_slot_buttons[i];        
        local texture = _G[buttonname.."CUI_Border"];
        
        local itemLink = GetInventoryItemLink("player", inv_slots[i]);
        
        if (texture) then
            if (itemLink) then
                texture:Show();
            
                local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
        
                if (itemName) then
                    
                    if (itemType ~= "Quest" and itemRarity) then
                        local r, g, b, hex = GetItemQualityColor(itemRarity);
                        CleanUI_SetItemBorderColor(texture, r, g, b);
                    else
                        CleanUI_SetItemBorderColor(texture, 1, 1, 1);
                    end
                end
            else
                texture:Hide();
            end
        end
	end
end

function CleanUI_SetItemBorderColor(texture, r, g, b)
    if (texture) then
        texture:SetVertexColor(r, g, b);
    end
end