local hoveredButton = nil;

function CleanUI_ChatOnEvent(self, event, ...)
    if (event == "MODIFIER_STATE_CHANGED") then
        CleanUI_UpdateButton(hoveredButton);
    end
end

function CleanUI_ChatHookEvents()
    if (CleanUIData.CHANGE_CHAT == 1) then
        hooksecurefunc("ChatEdit_DeactivateChat", CleanUI_ChatEdit_RestoreEditBoxAlpha);
        hooksecurefunc("ChatEdit_SetLastActiveWindow", CleanUI_ChatEdit_RestoreEditBoxAlpha);
        hooksecurefunc("ChatEdit_UpdateHeader", CleanUI_ChatEdit_UpdateHeader);
    end
end

function CleanUI_InitChat()

    if (CleanUIData.DISABLE_ALTARROWKEYMODE == 1) then
        CleanUI_DisableAltArrowKeyMode();
    end
    
    if (CleanUIData.CHANGE_CHAT == 1) then
        CleanUI_ChangeChat();
    end
end

function CleanUI_DisableAltArrowKeyMode()
    local i = 1;
    local editbox = "ChatFrame"..i.."EditBox";
    
    while ( _G[editbox] ~= nil ) do
        _G[editbox]:SetAltArrowKeyMode(nil);
        i = i + 1;
        editbox = "ChatFrame"..i.."EditBox";
    end
end

function CleanUI_ChatEdit_RestoreEditBoxAlpha(editBox)
	editBox:SetAlpha(1.0);
end

function CleanUI_ChatEdit_UpdateHeader(editBox)
	local type = editBox:GetAttribute("chatType");
	if ( not type ) then
		return;
	end
	
	local info = ChatTypeInfo[type];
	
	local header = _G[editBox:GetName().."Header"];
	
	if (header) then
        header:ClearAllPoints();
        header:SetPoint("LEFT", editBox, "LEFT", 6, 0);
        
        editBox:SetTextInsets(6 + header:GetWidth(), 6, 0, 0);
        editBox:SetBackdropBorderColor(info.r, info.g, info.b, 1.0);
    end
end

function CleanUI_ChangeChat()
    CleanUI_ChangeChatBackgroundTextures();
    CleanUI_ChangeChatBorders();
    CleanUI_ChangeChatButtonFrame();
    CleanUI_ChangeChatEditBox();
    
    CleanUI_AddChatExtensions();
end

function CleanUI_ChangeChatBackgroundTextures()
    for i = 1, 10 do        
        -- Textures
        _G["ChatFrame"..i.."Background"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."LeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."RightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopTexture"]:SetTexture(nil);
        
        _G["ChatFrame"..i.."ButtonFrameBackground"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopTexture"]:SetTexture(nil);
        
        _G["ChatFrame"..i.."EditBoxLeft"]:SetTexture(nil);
        _G["ChatFrame"..i.."EditBoxMid"]:SetTexture(nil);
        _G["ChatFrame"..i.."EditBoxRight"]:SetTexture(nil);
        
        _G["ChatFrame"..i.."EditBoxFocusLeft"]:SetTexture(nil);
        _G["ChatFrame"..i.."EditBoxFocusMid"]:SetTexture(nil);
        _G["ChatFrame"..i.."EditBoxFocusRight"]:SetTexture(nil);
    end
end

function CleanUI_ChangeChatBorders()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local buttonframe = _G["ChatFrame"..i.."ButtonFrame"];
        local editbox = _G["ChatFrame"..i.."EditBox"];
        
        -- tab
        
        -- Border        
        chatframe:SetBackdrop( { 
                bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
                insets = { left = -2, right = 0, top = 2, bottom = -2 }
            });
        chatframe:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
        
        local borderframe = _G["CUI_Chat"..i];
        if (not borderframe) then
            borderframe = CreateFrame("Frame", "CUI_Chat"..i, chatframe);
        end
        
        borderframe:SetBackdrop( { 
                edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16
            });
        
        borderframe:ClearAllPoints();
        borderframe:SetPoint("TOPLEFT", chatframe, "TOPLEFT", -6, 2);
        borderframe:SetPoint("BOTTOMRIGHT", chatframe, "BOTTOMRIGHT", 4, -6);
    end
end

function CleanUI_ChangeChatEditBox()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local editbox = _G["ChatFrame"..i.."EditBox"];
    
        -- EditBox
        editbox:SetBackdrop( { 
            bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
            edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
            insets = { left = 4, right = 4, top = 4, bottom = 4 }
        });
        editbox:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
        
        editbox:SetHeight(28);
        
        editbox:ClearAllPoints();
        editbox:SetPoint("TOPLEFT", chatframe, "BOTTOMLEFT", -6, -3);
        editbox:SetPoint("TOPRIGHT", chatframe, "BOTTOMRIGHT", 4, -3);
    end
end

function CleanUI_ChangeChatButtonFrame()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local buttonframe = _G["ChatFrame"..i.."ButtonFrame"];
        
        -- ButtonFrame
        buttonframe:SetBackdrop( { 
            bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
            edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
            insets = { left = 4, right = 4, top = 4, bottom = 4 }
        });
        buttonframe:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
        
        buttonframe:SetWidth(32);
        
        buttonframe:ClearAllPoints();
        buttonframe:SetPoint("TOPRIGHT", chatframe, "TOPLEFT", -3, 2);
        buttonframe:SetPoint("BOTTOMRIGHT", chatframe, "BOTTOMLEFT", -3, -31);
        
        -- Buttons        
        
        local bottomButton = _G["ChatFrame"..i.."ButtonFrameBottomButton"];
        local downButton = _G["ChatFrame"..i.."ButtonFrameDownButton"];
        local upButton = _G["ChatFrame"..i.."ButtonFrameUpButton"];
        local minimizeButton = _G["ChatFrame"..i.."ButtonFrameMinimizeButton"];
        
        bottomButton:ClearAllPoints();
        bottomButton:SetPoint("BOTTOM", buttonframe, "BOTTOM", 0, 35); 
        
        bottomButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom");
        bottomButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom-clicked");
        bottomButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom-disabled");
        
        downButton:ClearAllPoints();
        downButton:SetPoint("BOTTOM", bottomButton, "TOP", 0, -4); 
        
        downButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down");
        downButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down-clicked");
        downButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down-disabled");
        
        upButton:ClearAllPoints();
        upButton:SetPoint("BOTTOM", downButton, "TOP", 0, -4); 
        
        upButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up");
        upButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up-clicked");
        upButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up-disabled");
        
        minimizeButton:ClearAllPoints();
        minimizeButton:SetPoint("BOTTOM", upButton, "TOP", 0, -4); 
        
        minimizeButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize");
        minimizeButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize-clicked");
        minimizeButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize-disabled");
    end
    
    FriendsMicroButton:ClearAllPoints();
    FriendsMicroButton:SetPoint("BOTTOM", ChatFrameMenuButton, "TOP", 0, -150); 
    
    FriendsMicroButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet");
    FriendsMicroButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet-clicked");
    FriendsMicroButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet-disabled");
    
    ChatFrameMenuButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat");
    ChatFrameMenuButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat-clicked");
    ChatFrameMenuButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat-disabled");
end

function CleanUI_AddChatExtensions()
    if (CleanUIData.SHOW_CHAT_ACTIONBAR == 1) then
        CleanUI_AddChatActionBar();
    end
    
    if (CleanUIData.SHOW_CHAT_EMOTICONS == 1) then
        CleanUI_AddChatMenuEmoticons();
    end
end

function CleanUI_AddChatActionBar()        
    local borderframe = _G["CUI_ChatActionBarBorder"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUI_ChatActionBarBorder", ChatFrame1);
    end
    
    borderframe:SetHeight(23);

    borderframe:SetBackdrop( { 
        bgFile = CleanUIData.CUI_FRAME_BACKGROUND, 
        edgeFile = CleanUIData.CUI_FRAME_BORDER, tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    borderframe:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
        
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", ChatFrame1EditBox, "BOTTOMLEFT", -29, 3); 
    borderframe:SetPoint("TOPRIGHT", ChatFrame1EditBox, "BOTTOMRIGHT", 0, 3); 

    for i=1, 8, 1 do    
    
        local posx = -6-(i-1) * 16;
    
        local button = CreateFrame("Button", "CUI_CHAT_RAIDICON_"..i, borderframe, "UIPanelButtonTemplate");
        
        button:SetNormalTexture(CUI_RAIDICON_FILE);
        button:SetPushedTexture(CUI_RAIDICON_FILE);
        button:GetNormalTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        button:GetPushedTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);

        button:SetWidth(13);
        button:SetHeight(13);
        button:ClearAllPoints();
        button:SetPoint("RIGHT", borderframe, "RIGHT", posx, 0);
        
        button:HookScript("OnClick", function() CleanUI_ChatInsertText("{rt"..i.."}"); end);
    end 
    
    -- dice button
    local dicebutton = CreateFrame("Button", "CUI_CHAT_DICEBUTTON", borderframe, "UIPanelButtonTemplate");
    
    dicebutton:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up");
    dicebutton:SetPushedTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Down");
    dicebutton:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Highlight");
    
    dicebutton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    dicebutton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    dicebutton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    
    dicebutton:SetWidth(18);
    dicebutton:SetHeight(18);
    dicebutton:ClearAllPoints();
    dicebutton:SetPoint("LEFT", borderframe, "LEFT", 7, -1);
    dicebutton:HookScript("OnClick", function() RandomRoll(1, 100); end);
    
    -- GZ button
    local gzbutton = CreateFrame("Button", "CUI_CHAT_GZBUTTON", borderframe, "UIPanelButtonTemplate");
    gzbutton:SetWidth(30);
    gzbutton:SetHeight(18);
    gzbutton:ClearAllPoints();
    gzbutton:SetPoint("LEFT", borderframe, "LEFT", 30, 0);
    gzbutton:SetAttribute("TEXT", "GZ");
    CleanUI_UpdateButton(gzbutton);
    
    gzbutton:SetNormalTexture(nil);
    gzbutton:SetPushedTexture(nil);
    gzbutton:SetDisabledTexture(nil);
    gzbutton:SetHighlightTexture(nil);
    
    gzbutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_GZBUTTON"], CUI_CHAT_GZ); end);
    gzbutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_GZBUTTON"]); end);
    gzbutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_GZBUTTON"]); end);
    
    -- THX button
    local thxbutton = CreateFrame("Button", "CUI_CHAT_THXBUTTON", borderframe, "UIPanelButtonTemplate");
    thxbutton:SetWidth(30);
    thxbutton:SetHeight(18);
    thxbutton:ClearAllPoints();
    thxbutton:SetPoint("LEFT", gzbutton, "RIGHT", 2, 0);
    thxbutton:SetAttribute("TEXT", "THX");
    CleanUI_UpdateButton(thxbutton);
    
    thxbutton:SetNormalTexture(nil);
    thxbutton:SetPushedTexture(nil);
    thxbutton:SetDisabledTexture(nil);
    thxbutton:SetHighlightTexture(nil);
    
    thxbutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_THXBUTTON"], CUI_CHAT_THX); end);
    thxbutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_THXBUTTON"]); end);
    thxbutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_THXBUTTON"]); end);
end

function CleanUI_Great_OnClick(self, textarray)
   	local rnd = math.random(1, #textarray);
    local msg = textarray[rnd].text;
    local chatType = "GUILD";

    if (IsAltKeyDown() and self == hoveredButton) then
        if (GetNumRaidMembers() > 0) then
            chatType = "RAID";
        elseif (GetNumPartyMembers() > 0) then
            chatType = "PARTY";
        end
    else
        chatType = "GUILD";
    end
    
    SendChatMessage(msg, chatType);
end

function CleanUI_Great_OnEnter(self)
    hoveredButton = self;
    CleanUI_UpdateButton(self);
end

function CleanUI_Great_OnLeave(self)
    hoveredButton = nil;
    CleanUI_UpdateButton(self);
end

function CleanUI_UpdateButton(self)
    if (not self) then
        return;
    end

    local text = self:GetAttribute("TEXT");
    
    if (IsAltKeyDown() and self == hoveredButton) then
        if (GetNumRaidMembers() > 0) then
            self:SetText("|cffff7f3f"..text.."|r"); --raid
        elseif (GetNumPartyMembers() > 0) then
            self:SetText("|cff2090ff"..text.."|r"); --group
        end
    elseif (not hoveredButton) then
        self:SetText("|cffdddddd"..text.."|r"); --nothing
    else
        self:SetText("|cff20ff20"..text.."|r"); --guild
    end
end

function CleanUI_AddChatMenuEmoticons()
	UIMenu_AddButton(ChatMenu, "Emoticons", nil, nil, "CUI_EmoticonMenu");
	UIMenu_AutoSize(ChatMenu);
end

function CleanUI_InitEmoticonMenu(self)
	
	UIMenu_Initialize(self);
	UIMenu_AddButton(self, "White Smiley", nil, function(frame) CleanUI_ChatInsertText("☺") end);
	UIMenu_AddButton(self, "Black Smiley", nil, function(frame) CleanUI_ChatInsertText("☻") end);
	UIMenu_AddButton(self, "Sun", nil, function(frame) CleanUI_ChatInsertText("☼") end);
	UIMenu_AddButton(self, "Black heart", nil, function(frame) CleanUI_ChatInsertText("♥") end);
	UIMenu_AddButton(self, "Diamond", nil, function(frame) CleanUI_ChatInsertText("♦") end);
	UIMenu_AddButton(self, "Clove (Clover/Puppyfeet)", nil, function(frame) CleanUI_ChatInsertText("♣") end);
	UIMenu_AddButton(self, "Spade", nil, function(frame) CleanUI_ChatInsertText("♠") end);
	UIMenu_AddButton(self, "Musical Double Quarter Note", nil, function(frame) CleanUI_ChatInsertText("♫") end);
	UIMenu_AutoSize(self);
end

function CleanUI_ChatInsertText(text)
	ChatMenu:Hide();
	
    if (not DEFAULT_CHAT_FRAME.editBox:IsShown()) then
        DEFAULT_CHAT_FRAME.editBox:Show();
    end
    
	DEFAULT_CHAT_FRAME.editBox:SetFocus()
	DEFAULT_CHAT_FRAME.editBox:Insert(text);
end