local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local ldb = LibStub:GetLibrary("LibDataBroker-1.1");

function CleanUI_DataBrokerOnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "CleanUI") then            
		    if (MASQUE) then
                buttongroup = MASQUE:Group("CleanUI");
            end
        end
    end
end

function CleanUI_DataBrokerHookEvents()
end

function CleanUI_DataBrokerOnUpdate(self, elapsed)
end

function CleanUI_InitDataBroker()
    if (ldb) then
    
        -- neue Objekte
        ldb.RegisterCallback("CleanUI", "LibDataBroker_DataObjectCreated", CleanUI_DataBroker_DataObjectCreated);
        ldb.RegisterCallback("CleanUI", "LibDataBroker_AttributeChanged", CleanUI_DataBroker_AttributeChanged);
        
        -- schon erstellte Objekte
        for name, dataobj in ldb:DataObjectIterator() do
			CleanUI_DataBroker_AddObject(name, dataobj);
		end
		
		CleanUI_DataBroker_UpdateButtons();
    end
    
	CleanUIMinimapDataBrokerExpandButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand");
	CleanUIMinimapDataBrokerExpandButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand_down");
    
    if (CleanUIData.SHOW_DATABROKER == 1) then 
        _G["CleanUIMinimapDataBroker"]:Show();
    else
        _G["CleanUIMinimapDataBroker"]:Hide();
    end
end

function CleanUI_DataBroker_ShowButtons()
	CleanUIMinimapDataBrokerExpandButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\colapse");
	CleanUIMinimapDataBrokerExpandButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\colapse_down");

    local width, height = CleanUI_DataBroker_UpdateButtons();
    
    for name, button in pairs(db_buttons) do 
        if (CleanUIData.DataBroker_Disabled[name] ~= 1) then
            button:Show();
        else
            button:Hide();
        end
    end
    
    CleanUIMinimapDataBroker.Expanded = 1;
    
    CleanUIMinimapDataBroker:SetWidth(width);
    CleanUIMinimapDataBroker:SetHeight(height);
end

function CleanUI_DataBroker_HideButtons()
	CleanUIMinimapDataBrokerExpandButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand");
	CleanUIMinimapDataBrokerExpandButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand_down");

    for name, button in pairs(db_buttons) do 
        button:Hide();
    end
    
    CleanUIMinimapDataBroker.Expanded = nil;
    
    CleanUIMinimapDataBroker:SetWidth(32);
    CleanUIMinimapDataBroker:SetHeight(23);
end

function CleanUI_DataBroker_UpdateButtons()
    local buttonframe = _G["CleanUIMinimapDataBroker"];
    
    local width = 32;    
    local height = 23;
    
    local count = 0;    
    local row = 0;  
    local column = 0;
    
    local x_offset = -27;
    local y_offset = -27;
    
    local x, y;

    for name, dataobj in ldb:DataObjectIterator() do
        local button = db_buttons[name];
        
        if (button and CleanUIData.DataBroker_Disabled[name] ~= 1) then
            row = mod(count, CleanUIData.DATABROKER_COUNTROWS);            
            if (row == 0) then
                column = column + 1;
            end
            
            x = column * x_offset;
            y = row * y_offset - 7;
        
            _G[button:GetName().."Icon"]:SetTexture(dataobj.icon);
            
            button:ClearAllPoints();
            button:SetPoint("TOPRIGHT", buttonframe, "TOPRIGHT", x - 4, y);
            button:Hide();
            
            count = count + 1;
        end
    end
    
    if (count > 0) then
        width = 4 + width - column * x_offset;
        height = 10 - min(count, CleanUIData.DATABROKER_COUNTROWS) * x_offset;
    end
    
    if (CleanUIData.SHOW_DATABROKER == 1) then 
        _G["CleanUIMinimapDataBroker"]:Show();
    else
        _G["CleanUIMinimapDataBroker"]:Hide();
    end
    
    return width, height;
end

function CleanUI_DataBroker_DataObjectCreated(event, name, dataobj)
	CleanUI_DataBroker_AddObject(name, dataobj);
	CleanUI_DataBroker_UpdateButtons();
end

function CleanUI_DataBroker_AttributeChanged(event, name, key, value, dataobj)
	CleanUI_DataBroker_UpdateButtons();
end

function CleanUI_DataBroker_AddObject(name, dataobj)
    local buttonframe = _G["CleanUIMinimapDataBroker"];

    if (not buttonframe) then
        return;
    end
	
	if (dataobj.type == "launcher") then
	    -- create button
	    dataobj.cui_name = name;
	    
        local buttonname = "CUI_DataBroker_"..name;        
        local button = CreateFrame("Button", buttonname, buttonframe, "ActionButtonTemplate");
         
	    button:RegisterForClicks("AnyUp");
        
        button.dataobj = dataobj;
    
        button:SetWidth(24);
        button:SetHeight(24);
        button:HookScript("OnClick", dataobj.OnClick);
        button:HookScript("OnClick", CleanUI_HideMinimapButtons);
        
        if (dataobj.OnEnter) then
            button:HookScript("OnEnter", dataobj.OnEnter);
        elseif (dataobj.OnTooltipShow) then
            button:HookScript("OnEnter", CleanUI_DataBroker_OnEnter);
        end
        
        if (dataobj.OnLeave) then
            button:HookScript("OnLeave", dataobj.OnLeave);
        end
        
        button:HookScript("OnLeave", CleanUI_DataBroker_OnLeave);
        
        if (MASQUE and buttongroup) then
            buttongroup:AddButton(button);
        end
        
        db_buttons[name] = button;
    end
end

function CleanUI_DataBroker_OnEnter(self)
    local buttonframe = _G["CleanUIMinimapDataBroker"];
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOM");
    self.dataobj.OnTooltipShow(GameTooltip);
end

function CleanUI_DataBroker_OnLeave()
	GameTooltip:Hide();
end