local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CleanUI_DeBuffsOnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "CleanUI") then            
		    if (MASQUE) then
                buttongroup = MASQUE:Group("CleanUI");
            end
        end
    end
	if (event == "UNIT_AURA") then
	    CleanUI_CleanDeBuffs();
    end
end

function CleanUI_DeBuffsHookEvents()
end

function CleanUI_CleanDeBuffs() 
    if (MASQUE and buttongroup) then
        if (CleanUIData.BUTTONFACADE_BUFFS == 1) then
            CleanUI_SkinBuffs(); 
        end
    end
end

function CleanUI_DeBuffsOnUpdate(self, elapsed)
    for slot = 1, 84 do
        local shown = nil;
    
        if (CleanUIData.ACTIONBARS_DEBUFFS == 1 and cui_inCombat) then
            local type, globalID, subType = GetActionInfo(slot);
            
            if (globalID) then
                spellname, spellrank = GetSpellInfo(globalID);
                
                local target = "target";
                local modifiedClick = GetModifiedClick("FOCUSCAST");
                
                -- focus infos
                if (modifiedClick == "ALT" and IsAltKeyDown()) then
                    target = "focus";                
                elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
                    target = "focus";            
                elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
                    target = "focus";
                end
            
                if (spellname and spellrank) then
                    -- check buffs
                    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(target, spellname, spellrank);

                    if (name and unitCaster == "player") then
                        shown = CleanUI_UpdateDeBuffInfo(slot, name, duration, expirationTime);
                    end
                    
                    if (not shown) then
                        -- check debuffs
                        name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(target, spellname, spellrank);
                         
                        if (name and unitCaster == "player") then
                            shown = CleanUI_UpdateDeBuffInfo(slot, name, duration, expirationTime);
                        end
                    end
                end
            end
        end
        
        if (not shown) then    
            local button = CleanUI_GetButtonForSlot(slot);
                
            if (button) then       
                local statusbar = CleanUI_GetStatusBar(button);
                statusbar:Hide();
            end
        end
    end
end

function CleanUI_SkinBuffs()
    local button;
	    
	for i=1, BUFF_MAX_DISPLAY do
	    button = _G["BuffButton"..i];
	    if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
        end
	end
	
	for i=1, DEBUFF_MAX_DISPLAY do
	    button = _G["DebuffButton"..i];
	    if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
            
            if (CleanUIData.BIGGER_DEBUFFS) then
                -- resize debuffs (default = 30)
                button:SetWidth(42);
                button:SetHeight(42);
            end
        end
	end
end

function CleanUI_InitDeBuffs()
end

function CleanUI_UpdateDeBuffInfo(slot, name, duration, expirationTime)
    local button = CleanUI_GetButtonForSlot(slot);
    
    if (not button) then
        return nil;
    end
    
    local statusbar = CleanUI_GetStatusBar(button);
    
    statusbar:Show();
    
    local startTime = expirationTime - duration;
    statusbar:SetMinMaxValues(0, expirationTime-startTime);
    statusbar:SetValue(expirationTime-GetTime());
    
    return 1;
end

function CleanUI_GetButtonForSlot(slot)
    local button = nil;
    
    --[[
    ActionBar page 1: slots 1 to 12 -- Note exceptions below for other classes 
    ActionBar page 2: slots 13 to 24
    ActionBar page 3 (Right ActionBar): slots 25 to 36
    ActionBar page 4 (Right ActionBar 2): slots 37 to 48
    ActionBar page 5 (Bottom Right ActionBar): slots 49 to 60
    ActionBar page 6 (Bottom Left ActionBar): slots 61 to 72
    ]]--    
    
    if (slot >= 1 and slot <= 12) then
        button = _G["ActionButton"..slot];
    elseif (slot >= 13 and slot <= 24) then
        button = _G["BonusActionButton"..(slot-12)];
    elseif (slot >= 25 and slot <= 36) then
        button = _G["MultiBarRightButton"..(slot-24)];
    elseif (slot >= 37 and slot <= 48) then
        button = _G["MultiBarLeftButton"..(slot-36)];
    elseif (slot >= 49 and slot <= 60) then
        button = _G["MultiBarBottomRightButton"..(slot-48)];
    elseif (slot >= 61 and slot <= 72) then
        button = _G["MultiBarBottomLeftButton"..(slot-60)];
    elseif (slot >= 73 and slot <= 84) then
        button = _G["BonusActionButton"..(slot-72)];
    end
    
    return button;
end

function CleanUI_GetStatusBar(button)
    local name = button:GetName().."DeBuffBar";
    local statusbar = _G[name];
    
    if (not statusbar) then
        statusbar = CreateFrame("StatusBar", name, button, "CleanUIActionButtonDeBuffBarTemplate");
    end
    
    local width = button:GetWidth() - 2;
    local height = CleanUIData.ACTIONBARS_DEBUFFS_HEIGHT or 2;
    
    statusbar:SetWidth(width);
    statusbar:SetHeight(height);
    statusbar:ClearAllPoints();
    statusbar:SetPoint("BOTTOM", button, "BOTTOM", 0, 1);
    statusbar:SetStatusBarColor(CleanUIData.ACTIONBARS_DEBUFFS_R, CleanUIData.ACTIONBARS_DEBUFFS_G, CleanUIData.ACTIONBARS_DEBUFFS_B);

    return statusbar;
end
