function CleanUI_NameplatesOnEvent(self, event, ...)
end

function CleanUI_NameplatesHookEvents()	
end

function CleanUI_NameplatesOnUpdate(self, elapsed)
    if (CleanUIData.CHANGE_NAMEPLATES == 1) then
	    CleanUI_InitNewNameplates();
	end
end

function CleanUI_InitNewNameplates()
	for i=1, select("#", WorldFrame:GetChildren()) do
		local frame = select(i, WorldFrame:GetChildren());
		local region = frame:GetRegions();

		if(frame:GetName() and string.find(frame:GetName(), "NamePlate")) then
		    if (not frame.initialized) then
		        CleanUI_Nameplates_InitFrame(frame);
            end
            
		    CleanUI_Nameplates_UpdateFrame(frame);
        end
	end
end

function CleanUI_Nameplates_InitFrame(frame)
    frame.initialized = true;
    
    -- regions            
    local flash, nameplateBorder, nameplateGlow, name, level, skullIcon, raidIcon, eliteTexture = frame:GetRegions();
    frame.flash = flash;
    frame.nameplateBorder = nameplateBorder;
    frame.nameplateGlow = nameplateGlow;
    frame.name = name;
    frame.level = level;
    frame.skullIcon = skullIcon;
    frame.raidIcon = raidIcon;
    frame.eliteTexture = eliteTexture;
    
    flash:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\flash");
    
    nameplateBorder:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\border");  
    nameplateBorder:SetTexCoord(0, 1, 0, 1);
    
    nameplateGlow:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\glow");    
    nameplateGlow:SetTexCoord(0, 1, 0, 1);  
    
    raidIcon:SetWidth(16);
    raidIcon:SetHeight(16);
    raidIcon:ClearAllPoints();
    raidIcon:SetPoint("RIGHT", frame, "LEFT", 0, -9);
    
    eliteTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\elite");  
    
    name:SetFontObject("CuiFont_NamePlates_Name");
    level:SetFontObject("CuiFont_NamePlates_Name");
    
    -- statusbars
    local healthBar, castBar = frame:GetChildren();
    
    healthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    castBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    
    local statusbar, border, border_shield, _ = castBar:GetRegions();
    border:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\castbar");  
    border_shield:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\castbar-shield");  
    
    healthBar.text = healthBar:CreateFontString(nil, "ARTWORK");
    healthBar.text:SetFontObject("CuiFont_NamePlates_Health");
    healthBar.text:SetPoint("CENTER", healthBar, "CENTER", 5, 0);
    healthBar:HookScript("OnValueChanged", CleanUI_Nameplates_HealthOnValueChanged);
    healthBar:HookScript("OnMinMaxChanged", CleanUI_Nameplates_HealthOnValueChanged);
    
    frame.cb = border;
    frame.cb_shield = border_shield;
    
    -- hasfocus
    frame.focus = frame:CreateTexture(nil, "OVERLAY");
    frame.focus:ClearAllPoints();
    frame.focus:SetAllPoints(flash);
    frame.focus:SetTexCoord(flash:GetTexCoord());
    frame.focus:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\focus");
    frame.focus:Hide();
    
    -- ComboPoints
    frame.combopoints = CreateFrame("Frame", frame:GetName().."ComboPoints", frame);
    frame.combopoints:SetWidth(100);
    frame.combopoints:SetHeight(10);
    frame.combopoints:ClearAllPoints();
    frame.combopoints:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, -9);
    CleanUI_InitNameplateCombopoints(frame.combopoints);
    frame.combopoints:Hide();
end

function CleanUI_InitNameplateCombopoints(parent)	
    local last = nil;
	for i=1, 5 do
        local combo = _G[parent:GetName()..i];
        if (combo == nil) then
            combo = parent:CreateTexture(parent:GetName()..i, "OVERLAY");
            combo:SetWidth(6);
            combo:SetHeight(2);
            combo:SetTexture("Interface\\AddOns\\CleanUI\\skins\\combopoints\\"..i);
            combo:Hide();
        end
            
        if (last) then
            combo:ClearAllPoints();
            combo:SetPoint("LEFT", last, "RIGHT", 1, 0);
        else
            combo:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0); 
        end
        
        last = combo;
    end
end

function CleanUI_Nameplates_UpdateFrame(frame)
    if (CleanUIData.NAMEPLATES_HIGHLIGHTTARGET == 1 and UnitExists("target") and frame:GetAlpha() == 1.0) then
        frame.focus:Show();
    else
        frame.focus:Hide();
    end

    if (CleanUIData.NAMEPLATES_SHOWCOMBOPOINTS == 1 and UnitExists("target") and frame:GetAlpha() == 1.0) then
        frame.combopoints:Show();
        CleanUI_Nameplates_UpdateComboPoints(frame.combopoints);
    else
        frame.combopoints:Hide();
    end
end

function CleanUI_Nameplates_HealthOnValueChanged(frame, value)

    if (CleanUIData.NAMEPLATES_PERCENT ~= 1) then
		frame.text:SetText("");
		return;
    end

	local maxValue = select(2, frame:GetMinMaxValues());
	
	local healthtype = "percent";
	if(healthtype == "minmax") then
		frame.text:SetFormattedText("%d / %d", value, maxValue);
	elseif(healthtype == "percent") then
		frame.text:SetFormattedText("%d%%", value / maxValue * 100);
	else
		frame.text:SetText("");
	end
end

function CleanUI_Nameplates_UpdateComboPoints(parent)
    local combopoints = GetComboPoints("player", "target"); 
    
    for i=1, 5 do
        local combo = _G[parent:GetName()..i];     
        
        if (i > combopoints) then  
            combo:Hide();
        else
            combo:Show();
        end
    end
end

