function CleanUI_QuestOnEvent(self, event, ...)
end

function CleanUI_InitQuest()
    for i = 1, 10 do
        local questItem = _G["QuestInfoItem"..i];
        local questItemName = _G["QuestInfoItem"..i.."Name"];
        local questItemNameFrame = _G["QuestInfoItem"..i.."NameFrame"];
        
        questItemName:SetFontObject("GameFontHighlightSmall");	
        questItemName:ClearAllPoints(); 
        questItemName:SetPoint("LEFT", questItemNameFrame, "LEFT", 15, 5);
        
        --party pet name
        local price = questItem:CreateFontString("CUI_QuestItem"..i.."Price", "ARTWORK");
        price:SetFontObject("GameFontNormalSmall");
        price:ClearAllPoints(); 
        price:SetPoint("BOTTOMRIGHT", questItemNameFrame, "BOTTOMRIGHT", -15, 15);
        price:SetJustifyH("RIGHT");
        price:SetText("");
    end
end

function CleanUI_QuestHookEvents()
    hooksecurefunc("QuestInfo_Display", CleanUI_LoadQuestItemInfo);
	
	hooksecurefunc("QuestLog_Update", CleanUI_UpdateQuestInfo);
	hooksecurefunc(QuestLogScrollFrame, "update", CleanUI_UpdateQuestInfo);
end

function CleanUI_LoadQuestItemInfo()	
    for i = 1, 10 do
        local questItem = _G["QuestInfoItem"..i];
        local questItemPrice = _G["CUI_QuestItem"..i.."Price"];
        
        if (questItemPrice) then
            questItemPrice:SetText(ITEM_UNSELLABLE);
            
            if (questItem.objectType == "item" and questItem:IsVisible()) then
        
                local itemLink;
            
                if (QuestInfoFrame.questLog) then
                    itemLink = GetQuestLogItemLink(questItem.type, i);
                else
                    itemLink = GetQuestItemLink(questItem.type, i);
                end
                        
                if (itemLink) then
                    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
    
                    local moneyString = ITEM_UNSELLABLE;
                    if (itemSellPrice and itemSellPrice > 0) then
                        moneyString = "|cffffffff"..GetCoinTextureString(itemSellPrice).."|r";
                    end
                    
                    questItemPrice:SetText(moneyString);
                end
            end
        end
    end
end

function CleanUI_UpdateQuestInfo()

    local buttons = QuestLogScrollFrame.buttons;
    
    local scrollOffset = HybridScrollFrame_GetOffset(QuestLogScrollFrame);
    local numEntries = GetNumQuestLogEntries();
    local questIndex, questLogTitle;
    local questLogTitleText, level, isHeader, _;
    
    local tag_format = " [%s] %s";
    
    for i=1, #buttons do
    
        questIndex = i + scrollOffset;
        questLogTitle = buttons[i];
        questNormalText = questLogTitle.normalText;
		--questTitleTag = questLogTitle.tag;
        
        if questIndex <= numEntries then
            questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
            
            if (not isHeader) and questLogTitleText then
            
            	local info = ""..level;
	
				if (suggestedGroup > 0) then
					info = info.."G"..suggestedGroup;
				end
            
                local text = tag_format:format(info, questLogTitleText);
                
                if (isComplete or numObjectives == 0) then
                	text = "|cff3399ff"..text.."|r";
                end
                
                questLogTitle:SetText(text);
                QuestLogTitleButton_Resize(questLogTitle);
            end
        end
    end
end