
local srcNames = {};
local damageDone = {};
local dps = {};
local maxDps = {};
local combatStartTime = {};
local lastShotTime = {};
local displayDps = {};
local dpsOfMaxDps = {};

local events = {
	SWING_DAMAGE = true,
	RANGE_DAMAGE = true,
	SPELL_DAMAGE = true,
	SPELL_PERIODIC_DAMAGE = true,
	DAMAGE_SHIELD = true,
	DAMAGE_SPLIT = true,
}

function CleanUI_UnitThreadOnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "CleanUI") then
        end
    end
    if (event == "UNIT_THREAT_LIST_UPDATE" or event == "UNIT_THREAT_SITUATION_UPDATE") then
        CleanUI_UnitThreadUpdate();
    end
    if (event == "PLAYER_REGEN_ENABLED") then
        CleanUI_StopCombat();
    end
    if (event == "PLAYER_REGEN_DISABLED") then
        CleanUI_StartCombat();
    end
    if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
        local timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId, spellName, spellSchool, amount = ...;
        CleanUI_CalculateDps(timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount);
    end
end

function CleanUI_InitThreatDataFrame()
    CleanUIPlayerDpsFrameStatusBar:ClearAllPoints(); 
    CleanUIPlayerDpsFrameStatusBar:SetPoint("TOPLEFT", CleanUIPlayerDpsFrame, "TOPLEFT", 5, -5);
    CleanUIPlayerDpsFrameStatusBar:SetPoint("BOTTOMRIGHT", CleanUIPlayerDpsFrame, "BOTTOMRIGHT", -5, 5);
end


local UNIT_THREAT_UPDATE_WAIT = 0.5;
local _UnitThreadWaitTimeSinceLast = 0;
function CleanUI_UnitThreadOnUpdate(self, elapsed)

    if (_UnitThreadWaitTimeSinceLast <= 0) then
        CleanUI_UnitThreadUpdate();
        _UnitThreadWaitTimeSinceLast = UNIT_THREAT_UPDATE_WAIT;
    else
        _UnitThreadWaitTimeSinceLast = _UnitThreadWaitTimeSinceLast - elapsed;
    end
end

function CleanUI_StartCombat()
    srcNames = {};
    damageDone = {};
    dps = {};
    maxDps = {};
    combatStartTime = {};
    lastShotTime = {};
    displayDps = {};
    dpsOfMaxDps = {};
    
    cui_inCombat = 1;
end

function CleanUI_StopCombat()
    cui_inCombat = nil;
end

function CleanUI_CalculateDps(timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount)
    
    if (CleanUIData.SHOW_DPS ~= 1) then
        return;
    end
    
    if (not cui_inCombat) then
        return;
    end
    
    if (not srcName) then
	    return;
	end
    
    if (not events[eventType]) then
	    return;
	end
    
	local owner = CleanUI_GetOwnerName(srcGUID);
	
	if (owner) then
	    if (CleanUIData.MERGEPETDPSDATA ~= 1) then
	        srcName = srcName.." ("..owner..")";
	    else
	        srcName = "+"..owner;
	    end
	end
    
    if (not srcNames[srcName]) then
	    srcNames[srcName] = srcName;
	end
	
	if (eventType == "SWING_DAMAGE") then
		amount = spellId;
	end
	
	-- collect values	
	if (amount and amount > 0) then
	 	local newTime = GetTime();
		local cst = combatStartTime[srcName] or newTime;
		local lst = newTime;
		local dd = (damageDone[srcName] or 0) + amount;
		
	    local diffTime = lst - cst;
		
		combatStartTime[srcName] = cst;
		lastShotTime[srcName] = lst;
		damageDone[srcName] = dd;
	end
	
	CleanUI_CalculateBarValues(srcName);
end

function CleanUI_CalculateBarValues(srcName)
	-- calculate now
	local value = (CleanUI_CalculateDpsForSource(srcName) or 0);	
	local max_value = (maxDps[srcName] or 0);
	
	-- pets
	if (CleanUIData.MERGEPETDPSDATA == 1) then
        local pet_value = (CleanUI_CalculateDpsForSource("+"..srcName) or 0);	
        local pet_max_value = (maxDps["+"..srcName] or 0);
        
        value = value + pet_value;
        max_value = max_value + pet_max_value;
	end
	
	local str_value = "";
	local str_maxvalue = "";
	
	if (value > 0) then
	    str_value = format("%.2f", value);
	else
	    str_value = "";
	    value = 0;
	end
	
	if (max_value > 0) then
	    str_maxvalue = format("%.2f", max_value);
	end
	
	displayDps[srcName] = value;
	dpsOfMaxDps[srcName] = str_value.."/"..str_maxvalue;
	
	if (srcName == GetUnitName("player")) then
	    CleanUI_ActualizeDpsBar(value, max_value, str_value);
	end
end

function CleanUI_GetOwnerName(petGUID)
    local pet;
    
    pet = UnitGUID("pet");
    if (pet and pet == petGUID) then
        return UnitName("player");
    end
    
    for i = 1, GetNumRaidMembers() do
        pet = UnitGUID("raidpet"..i);
        if (pet and pet == petGUID) then
            return GetUnitName("raid"..i);
        end
    end
    
    for i = 1,4 do
        pet = UnitGUID("partypet"..i);
        if (pet and pet == petGUID) then
            return GetUnitName("party"..i) ;
        end
    end
    
    return nil;
end

function CleanUI_CalculateDpsForSource(srcName)       
    
    if (not lastShotTime[srcName]) then
	    return;
	end	
    
    if (not combatStartTime[srcName]) then
	    return;
	end	
    
    if (not damageDone[srcName]) then
	    return;
	end	
	
	local diffTime = lastShotTime[srcName] - combatStartTime[srcName];
    
    if (diffTime == 0) then
	    return;
	end	
	
	dps[srcName] = damageDone[srcName] / diffTime;
	
	if (dps[srcName] > (maxDps[srcName] or 0)) then
	    maxDps[srcName] = dps[srcName];
	end
	
	return dps[srcName];	
end

function CleanUI_ActualizeDpsBar(value, max_value, str_value)       
	CleanUIPlayerDpsFrameStatusBar:SetMinMaxValues(0, max_value);	
    CleanUIPlayerDpsFrameStatusBar:SetValue(value);	
	CleanUIPlayerDpsFrameStatusBarDps:SetText(str_value);
end

function CleanUI_DpsOnEnter(frame)    
    CleanUIPlayerDpsFrameStatusBarDps:Show();
    CleanUI_ShowDpsTooltip(frame);
end

function CleanUI_DpsOnLeave(frame) 
    if (CleanUIData.SHOW_DPSINFO ~= 1) then
        CleanUIPlayerDpsFrameStatusBarDps:Hide();
    end
    
	GameTooltip:Hide();      
end

function CleanUI_ShowDpsTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetText("|c333399ffDPS|r");
    
    local orderedNames = {};
    
    for index, srcName in pairs(srcNames) do     
        if (displayDps[srcName]) then
            orderedNames[displayDps[srcName]] = srcName;
        end
    end
    
    local group_dps = 0;
    
    local index = 1;
    for dps, srcName in pairsByKeysDesc(orderedNames) do 
        local startPos, _, _, _ = string.find(srcName, "+", 1);
        if (startPos ~= 1) then
        
            -- group dps
            if (CleanUI_IsInParty(srcName)) then
                group_dps = group_dps + displayDps[srcName];
            elseif (CleanUI_IsInRaid(srcName)) then
                group_dps = group_dps + displayDps[srcName];
            end
            
            if (srcName == UnitName("player")) then
                group_dps = group_dps + displayDps[srcName];
            end
       
            -- unit dps
            local name = HIGHLIGHT_FONT_COLOR_CODE..(srcName or "")..FONT_COLOR_CODE_CLOSE;
            
            if (srcName == UnitName("player")) then
                name = ORANGE_FONT_COLOR_CODE..(srcName or "")..FONT_COLOR_CODE_CLOSE;
            end
        
            GameTooltip:AddDoubleLine(index..": "..name, (dpsOfMaxDps[srcName] or ""), nil, nil, nil, nil, nil, nil);
            index = index + 1;
        end
    end
    
    if (group_dps) then
        GameTooltip:AddLine(" ");
        GameTooltip:AddDoubleLine(HIGHLIGHT_FONT_COLOR_CODE..GROUP..FONT_COLOR_CODE_CLOSE, (format("%.2f", group_dps) or ""), nil, nil, nil, nil, nil, nil);
    end
        
    GameTooltip:Show();
end

function CleanUI_IsInParty(name)

    if (GetNumPartyMembers() == 0) then
        return false;
    end

    for i = 1, GetNumPartyMembers() do
        local unitname = GetUnitName("party"..i);
        if (unitname == name) then
            return true;
        end
    end
    
    return false;
end

function CleanUI_IsInRaid(name)
    if (name == UnitName("player")) then
        return false;
    end

    if (GetNumRaidMembers() < 2) then
        return false;
    end
    
    for i = 1, GetNumRaidMembers() do
        local unitname = GetUnitName("raid"..i);
        if (unitname == name) then
            return true;
        end
    end
    
    return false;
end

function pairsByKeysAsc(t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, function(a,b) return a<b end)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end

function pairsByKeysDesc(t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, function(a,b) return a>b end)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end

function CleanUI_UnitThreadUpdate()       
    if (not cui_unitframes_loaded) then
        return;
    end
    
    CleanUI_UpdateUnitFrameAggroBars();
    
    CleanUI_PlayerInfo();
    CleanUI_TargetInfo();  
    CleanUI_FocusInfo();  
    
    CleanUI_UnitIsDeadOrGhost();
    CleanUI_UnitIsOnline();
    
 	CleanUI_RaidIconUpdate();
end

function CleanUI_UpdateUnitFrameAggroBars()   
    
    -- player
    local isTanking, status, percentage = UnitDetailedThreatSituation("player", "target");
    
    if (percentage == nil) then
        percentage = 0;
    end
    
    if (percentage > 0) then
        CUI_PlayerAggroBarInfo:SetText(format("%d", percentage).." %");
    else
        CUI_PlayerAggroBarInfo:SetText("");
    end
    
    if (percentage > 100) then
        percentage = 100;
    end
    
    CUI_PlayerAggroBar:SetValue(percentage);   
    
    -- pet
    if (UnitExists("pet")) then
        if (UnitExists("pettarget")) then
            -- set bordercolor, if pet is tanking another target
            local isTanking, status, percentage = UnitDetailedThreatSituation("pet", "pettarget");
            local r, g, b = GetThreatStatusColor(status); 
            
            PetFrame:SetBackdropBorderColor(r, g, b, 1.0); 
        else
            PetFrame:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
        end
        
        if (UnitExists("playertarget")) then
            -- set aggro bar value for player target
            local isTanking, status, percentage = UnitDetailedThreatSituation("pet", "playertarget");
            
            if (percentage == nil) then
                percentage = 0;
            end
            
            if (percentage > 100) then
                percentage = 100;
            end
            
            _G["CUI_PetAggroBar"]:SetValue(percentage);     
        else
            _G["CUI_PetAggroBar"]:SetValue(0);     
        end 
    end
    
    -- party
    for i = 1, 4 do
        local id = "party"..i;
        if (UnitExists(id)) then
            if (UnitExists(id.."target")) then
                -- set bordercolor, if party# is tanking another target
                local isTanking, status, percentage = UnitDetailedThreatSituation(id, id.."target");
                local r, g, b = GetThreatStatusColor(status); 
                
                _G["PartyMemberFrame"..i]:SetBackdropBorderColor(r, g, b, 1.0); 
            else
                _G["PartyMemberFrame"..i]:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
            end
            
            if (UnitExists("playertarget")) then
                -- set aggro bar value for player target
                local isTanking, status, percentage = UnitDetailedThreatSituation(id, "playertarget");
                
                if (percentage == nil) then
                    percentage = 0;
                end
                
                if (percentage > 100) then
                    percentage = 100;
                end
                
                _G["CUI_Party"..i.."AggroBar"]:SetValue(percentage);     
            else
                _G["CUI_Party"..i.."AggroBar"]:SetValue(0);     
            end
        end
    
        local pet = "partypet"..i;

        if (UnitExists(pet)) then
            local petmanabar = _G["CUI_Party"..i.."PetManaBar"];
            
            if (petmanabar) then
                local power = 100 * UnitPower(pet) / UnitPowerMax(pet);
            
                if (UnitPowerMax(pet) == 0) then
                    power = 0;
                end
            
                petmanabar:SetValue(power);
            end
            
            if (UnitExists(pet.."target")) then
                -- set aggro bar value for party pet target
                local isTanking, status, percentage = UnitDetailedThreatSituation(pet.."target", "playertarget");
                
                if (percentage == nil) then
                    percentage = 0;
                end
                
                if (percentage > 100) then
                    percentage = 100;
                end
                
                _G["CUI_Party"..i.."PetAggroBar"]:SetValue(percentage);   
            else
                _G["CUI_Party"..i.."PetAggroBar"]:SetValue(0);  
            end
        else
            _G["PartyMemberFrame"..i.."PetFrame"]:Hide();
        end
    end
end

function CleanUI_PlayerInfo()
    
    if (CleanUIData.ACTIONBARS_SHOWSTATUS == 1) then 
    
        local health = 100 * UnitHealth("player") / UnitHealthMax("player");
        local power = 100 * UnitPower("player") / UnitPowerMax("player");
        
        if (UnitPowerMax("player") == 0) then
            power = 0;
        end
        
        CleanUIActionButtonsHealthStatusBar:SetValue(health);
        CleanUIActionButtonsPowerStatusBar:SetValue(power);
 		
        CleanUIActionButtonsHealthStatusBarText:SetText(format("%d", health).." %");
        CleanUIActionButtonsPowerStatusBarText:SetText(format("%d", power).." %");
        
        local powerType, powerToken, altR, altG, altB = UnitPowerType("player");
        local info = PowerBarColor[powerToken];
        if ( info ) then
            if ( not CleanUIActionButtonsPowerStatusBar.lockColor ) then
                CleanUIActionButtonsPowerStatusBar:SetStatusBarColor(info.r, info.g, info.b);
            end
        else
            if ( not altR) then
                info = PowerBarColor[powerType] or PowerBarColor["MANA"];
            else
                if ( not manaBar.lockColor ) then
                    CleanUIActionButtonsPowerStatusBar:SetStatusBarColor(altR, altG, altB);
                end
            end
        end
    end
end

function CleanUI_TargetInfo()   
    if (UnitExists("target")) then
        local r, g, b = GameTooltip_UnitColor("target");     
        if (not UnitPlayerControlled("target") and UnitIsTapped("target") and not UnitIsTappedByPlayer("target") and not UnitIsTappedByAllThreatList("target")) then
            r, g, b = 0.5, 0.5, 0.5;
            CUI_TargetTabbedInfo:SetText("tapped");
        else
            CUI_TargetTabbedInfo:SetText("");
        end
        
        TargetFrameTextureFrameName:SetTextColor(r, g, b);
        
        r, g, b = GetThreatStatusColor(status);
        PlayerFrame:SetBackdropBorderColor(r, g, b, 1.0);
    
        local t_health = 100 * UnitHealth("target") / UnitHealthMax("target");
 		local t_power = 100 * UnitPower("target") / UnitPowerMax("target");
 		
 		if (UnitPowerMax("target") == 0) then
 		    t_power = 0;
 		end
 		
        CUI_TargetHealthBarPercent:SetText(format("%d", t_health).." %");
        CUI_TargetManaBarPercent:SetText(format("%d", t_power).." %");
    else
        PlayerFrame:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
        
        CUI_TargetHealthBarPercent:SetText("");
        CUI_TargetManaBarPercent:SetText("");
    end
    
    if (CUI_TargetFrameBorder1) then
        local targetClassification = UnitClassification("target");
        
        if (targetClassification == "rareelite" or targetClassification == "elite") then
            CUI_TargetFrameEliteTexture:Show();
            CUI_TargetFrameRareTexture:Hide();
            TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
            TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
            CUI_TargetHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
            CUI_TargetManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
        elseif (targetClassification == "rare") then
            CUI_TargetFrameEliteTexture:Hide();
            CUI_TargetFrameRareTexture:Show();
            TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
            TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
            CUI_TargetHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
            CUI_TargetManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
        else
            CUI_TargetFrameEliteTexture:Hide();
            CUI_TargetFrameRareTexture:Hide();
            TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
            TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
            CUI_TargetHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
            CUI_TargetManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
        end
    end
    
    if (CUI_TargetInfo) then
        local creatureType = UnitCreatureType("target");
        local creatureFamily = UnitCreatureFamily("target");
        local localizedClass, englishClass = UnitClass("target");
        
		local classColor = RAID_CLASS_COLORS[englishClass];

        if (localizedClass == UnitName("target")) then
            localizedClass = nil;
        end
        
        local targetinfo = "";
         
        if (creatureType) then
            targetinfo = creatureType;
        end
        
        if (localizedClass and localizedClass ~= "") then
		    local classInfo = format("|cff%.2x%.2x%.2x%s|r", classColor.r * 255, classColor.g * 255, classColor.b * 255, localizedClass);
            targetinfo = targetinfo..", "..classInfo;
        end
        
        if (creatureFamily and creatureFamily ~= "") then
            targetinfo = targetinfo..", "..creatureFamily;
        end
        
        CUI_TargetInfo:SetText(targetinfo);
    end
end

function CleanUI_FocusInfo()   
    if (UnitExists("focus")) then
        local r, g, b = GameTooltip_UnitColor("focus");
        
        if (not UnitPlayerControlled("focus") and UnitIsTapped("focus") and not UnitIsTappedByPlayer("focus") and not UnitIsTappedByAllThreatList("focus")) then
            r, g, b = 0.5, 0.5, 0.5;
            CUI_FocusTabbedInfo:SetText("tapped");
        else
            CUI_FocusTabbedInfo:SetText("");
        end

        FocusFrameTextureFrameName:SetTextColor(r, g, b);
        
        local f_health = 100 * UnitHealth("focus") / UnitHealthMax("focus");
 		local f_power = 100 * UnitPower("focus") / UnitPowerMax("focus");
 		
 		if (UnitPowerMax("focus") == 0) then
 		    f_power = 0;
 		end
 		
        CUI_FocusHealthBarPercent:SetText(format("%d", f_health).." %");
        CUI_FocusManaBarPercent:SetText(format("%d", f_power).." %");
    else
        CUI_FocusHealthBarPercent:SetText("");
        CUI_FocusManaBarPercent:SetText("");
    end
    
    if (CUI_FocusFrameBorder1) then
        local focusClassification = UnitClassification("focus");
        
        if (focusClassification == "rareelite" or focusClassification == "elite") then
            CUI_FocusFrameEliteTexture:Show();
            CUI_FocusFrameRareTexture:Hide();
            FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            CUI_FocusHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            CUI_FocusManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        elseif (focusClassification == "rare") then
            CUI_FocusFrameEliteTexture:Hide();
            CUI_FocusFrameRareTexture:Show();
            FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            CUI_FocusHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
            CUI_FocusManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        else
            CUI_FocusFrameEliteTexture:Hide();
            CUI_FocusFrameRareTexture:Hide();
            FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
            FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 1.0, 1.0, 1.0);
            CUI_FocusHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
            CUI_FocusManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
        end
    end
    
    if (CUI_FocusInfo) then
        local creatureType = UnitCreatureType("focus");
        local creatureFamily = UnitCreatureFamily("focus");
        local localizedClass, englishClass = UnitClass("focus");
        
		local classColor = RAID_CLASS_COLORS[englishClass];
        
        if (localizedClass == UnitName("focus")) then
            localizedClass = nil;
        end
         
        local focusinfo = "";
         
        if (creatureType) then
            focusinfo = creatureType;
        end
        
        if (localizedClass and localizedClass ~= "") then
		    local classInfo = format("|cff%.2x%.2x%.2x%s|r", classColor.r * 255, classColor.g * 255, classColor.b * 255, localizedClass);
            focusinfo = focusinfo..", "..classInfo;
        end
        
        if (creatureFamily and creatureFamily ~= "") then
            focusinfo = focusinfo..", "..creatureFamily;
        end
                
        CUI_FocusInfo:SetText(focusinfo);
    end
end

function CleanUI_UnitIsDeadOrGhost()   
    if (UnitIsDeadOrGhost("player")) then
        PlayerFrameHealthBar:SetValue(0);
        PlayerFrameManaBar:SetValue(0);
    end
    
    if (UnitIsDeadOrGhost("pet")) then
        PetFrameHealthBar:SetValue(0);
        PetFrameManaBar:SetValue(0);
    else
    end
    
    if (UnitIsDeadOrGhost("target")) then
        TargetFrameHealthBar:SetValue(0);
        TargetFrameManaBar:SetValue(0);
    else
    end
    
    if (UnitIsDeadOrGhost("focus")) then
        FocusFrameHealthBar:SetValue(0);
        FocusFrameManaBar:SetValue(0);
    else
    end
    
    if (UnitIsDeadOrGhost("targettarget")) then
        TargetFrameToTHealthBar:SetValue(0);
        TargetFrameToTManaBar:SetValue(0);
    else
    end
    
    if (UnitIsDeadOrGhost("targetfocus")) then
        TargetofFocusHealthBar:SetValue(0);
        TargetofFocusManaBar:SetValue(0);
    else
    end
end

function CleanUI_UnitIsOnline()   
    if (UnitIsAFK("player") or UnitIsConnected("player") == nil) then
		PlayerFrameHealthBar:SetStatusBarColor(0.5, 0.5, 0.5);
    else
		PlayerFrameHealthBar:SetStatusBarColor(0.0, 1.0, 0.0);
    end
    
    if (UnitIsAFK("target") or UnitIsConnected("target") == nil) then
		TargetFrameHealthBar:SetStatusBarColor(0.5, 0.5, 0.5);
    else
		TargetFrameHealthBar:SetStatusBarColor(0.0, 1.0, 0.0);
    end
    
    if (UnitIsAFK("targettarget") or UnitIsConnected("targettarget") == nil) then
		TargetFrameToTHealthBar:SetStatusBarColor(0.5, 0.5, 0.5);
    else
		TargetFrameToTHealthBar:SetStatusBarColor(0.0, 1.0, 0.0);
    end
    
    if (CUI_TargetInfo) then
        if (UnitIsAFK("target")) then
            CUI_TargetInfo:SetText(AFK);
        elseif (UnitIsConnected("target") == nil) then
            CUI_TargetInfo:SetText(PLAYER_OFFLINE);
        end
    end
    
    if (UnitIsAFK("focus") or UnitIsConnected("focus") == nil) then
		FocusFrameHealthBar:SetStatusBarColor(0.5, 0.5, 0.5);
    else
		FocusFrameHealthBar:SetStatusBarColor(0.0, 1.0, 0.0);
    end
    if (UnitIsAFK("focustarget") or UnitIsConnected("focustarget") == nil) then
		FocusFrameToTHealthBar:SetStatusBarColor(0.5, 0.5, 0.5);
    else
		FocusFrameToTHealthBar:SetStatusBarColor(0.0, 1.0, 0.0);
    end
    
    if (CUI_FocusInfo) then
        if (UnitIsAFK("focus")) then
            CUI_FocusInfo:SetText(AFK);
        elseif (UnitIsConnected("focus") == nil) then
            CUI_FocusInfo:SetText(PLAYER_OFFLINE);
        end
    end
end